@@ -150,8 +150,13 @@ void MP4Integer8Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = %u (0x%02x)\n", 
-		m_name, m_values[index], m_values[index]);
+	if (index != 0) 
+	  fprintf(pFile, "%s[%u] = %u (0x%02x)\n", 
+		  m_name, index, m_values[index], m_values[index]);
+	else
+	  fprintf(pFile, "%s = %u (0x%02x)\n", 
+		  m_name, m_values[index], m_values[index]);
+	fflush(pFile);
 }
 
 void MP4Integer16Property::Dump(FILE* pFile, u_int8_t indent,
@@ -161,8 +166,13 @@ void MP4Integer16Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = %u (0x%04x)\n", 
-		m_name, m_values[index], m_values[index]);
+	if (index != 0) 
+	  fprintf(pFile, "%s[%u] = %u (0x%04x)\n", 
+		  m_name, index, m_values[index], m_values[index]);
+	else
+	  fprintf(pFile, "%s = %u (0x%04x)\n", 
+		  m_name, m_values[index], m_values[index]);
+	fflush(pFile);
 }
 
 void MP4Integer24Property::Dump(FILE* pFile, u_int8_t indent,
@@ -172,8 +182,13 @@ void MP4Integer24Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = %u (0x%06x)\n", 
-		m_name, m_values[index], m_values[index]);
+	if (index != 0) 
+	  fprintf(pFile, "%s[%u] = %u (0x%06x)\n", 
+		  m_name, index, m_values[index], m_values[index]);
+	else
+	  fprintf(pFile, "%s = %u (0x%06x)\n", 
+		  m_name, m_values[index], m_values[index]);
+	fflush(pFile);
 }
 
 void MP4Integer32Property::Dump(FILE* pFile, u_int8_t indent,
@@ -183,8 +198,13 @@ void MP4Integer32Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = %u (0x%08x)\n", 
+	if (index != 0) 
+	  fprintf(pFile, "%s[%u] = %u (0x%08x)\n", 
+		  m_name, index, m_values[index], m_values[index]);
+	else
+	  fprintf(pFile, "%s = %u (0x%08x)\n", 
 		m_name, m_values[index], m_values[index]);
+	fflush(pFile);
 }
 
 void MP4Integer64Property::Dump(FILE* pFile, u_int8_t indent,
@@ -194,9 +214,14 @@ void MP4Integer64Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, 
-		"%s = "U64" (0x%016"X64F")\n", 
-		m_name, m_values[index], m_values[index]);
+	if (index != 0) 
+	  fprintf(pFile, "%s[%u] = "U64" (0x%016"X64F")\n", 
+		  m_name, index, m_values[index], m_values[index]);
+	else
+	  fprintf(pFile, 
+		  "%s = "U64" (0x%016"X64F")\n", 
+		  m_name, m_values[index], m_values[index]);
+	fflush(pFile);
 }
 
 // MP4BitfieldProperty
@@ -229,9 +254,15 @@ void MP4BitfieldProperty::Dump(FILE* pFile, u_int8_t indent,
 	if (hexWidth == 0 || (m_numBits % 4)) {
 		hexWidth++;
 	}
+	if (index != 0) 
 	fprintf(pFile, 
-		"%s = "U64" (0x%0*"X64F") <%u bits>\n", 
-		m_name, m_values[index], (int)hexWidth, m_values[index], m_numBits);
+		"%s[%u] = "U64" (0x%0*"X64F") <%u bits>\n", 
+		m_name, index, m_values[index], (int)hexWidth, m_values[index], m_numBits);
+	else 
+	  fprintf(pFile, 
+		  "%s = "U64" (0x%0*"X64F") <%u bits>\n", 
+		  m_name, m_values[index], (int)hexWidth, m_values[index], m_numBits);
+	fflush(pFile);
 }
 
 // MP4Float32Property
@@ -271,8 +302,13 @@ void MP4Float32Property::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = %f\n", 
-		m_name, m_values[index]);
+	if (index != 0) 
+  	  fprintf(pFile, "%s[%u] = %f\n", 
+		  m_name, index, m_values[index]);
+	else
+ 	  fprintf(pFile, "%s = %f\n", 
+		  m_name, m_values[index]);
+	fflush(pFile);
 }
 
 // MP4StringProperty
@@ -369,11 +405,17 @@ void MP4StringProperty::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
+	char indexd[32];
+	if (index != 0) {
+	  snprintf(indexd, 32, "[%u]", index);
+	} else indexd[0] = '\0';
+
 	if (m_useUnicode) {
-		fprintf(pFile, "%s = %ls\n", m_name, (wchar_t*)m_values[index]);
+	  fprintf(pFile, "%s%s = %ls\n", m_name, indexd, (wchar_t*)m_values[index]);
 	} else {
-		fprintf(pFile, "%s = %s\n", m_name, m_values[index]);
+	  fprintf(pFile, "%s%s = %s\n", m_name, indexd, m_values[index]);
 	}
+	fflush(pFile);
 }
 
 // MP4BytesProperty
@@ -418,8 +460,12 @@ void MP4BytesProperty::SetValue(const u_int8_t* pValue, u_int32_t valueSize,
 	}
 	if (m_fixedValueSize) {
 		if (valueSize > m_fixedValueSize) {
-			throw new MP4Error("value size exceeds fixed value size",
-				"MP4BytesProperty::SetValue");
+			throw new MP4Error("%s.%s value size %d exceeds fixed value size %d",
+					   "MP4BytesProperty::SetValue",
+					   GetParentAtom()->GetType(),
+					   GetName(),
+					   valueSize, 
+					   m_fixedValueSize);
 		}
 		if (m_values[index] == NULL) {
 			m_values[index] = (u_int8_t*)MP4Calloc(m_fixedValueSize);
@@ -487,7 +533,9 @@ void MP4BytesProperty::Dump(FILE* pFile, u_int8_t indent,
 		return;
 	}
 	Indent(pFile, indent);
-	fprintf(pFile, "%s = <%u bytes> ", m_name, m_valueSizes[index]);
+	fputs(m_name, pFile);
+	if (index != 0) fprintf(pFile, "[%u]", index);
+	fprintf(pFile, " = <%u bytes> ", m_valueSizes[index]);
 	for (u_int32_t i = 0; i < m_valueSizes[index]; i++) {
 		if ((i % 16) == 0 && m_valueSizes[index] > 16) {
 			fprintf(pFile, "\n");
@@ -496,15 +544,14 @@ void MP4BytesProperty::Dump(FILE* pFile, u_int8_t indent,
 		fprintf(pFile, "%02x ", m_values[index][i]);
 	}
 	fprintf(pFile, "\n");
+	fflush(pFile);
 }
 
 // MP4TableProperty
 
-MP4TableProperty::MP4TableProperty(char* name, MP4Property* pCountProperty)
+MP4TableProperty::MP4TableProperty(char* name, MP4IntegerProperty* pCountProperty)
 	: MP4Property(name) 
 {
-	ASSERT(pCountProperty->GetType() == Integer8Property
-		|| pCountProperty->GetType() == Integer32Property);
 	m_pCountProperty = pCountProperty;
 	m_pCountProperty->SetReadOnly();
 }
@@ -629,7 +676,14 @@ void MP4TableProperty::Write(MP4File* pFile, u_int32_t index)
 
 	u_int32_t numEntries = GetCount();
 
-	ASSERT(m_pProperties[0]->GetCount() == numEntries);
+	if (m_pProperties[0]->GetCount() != numEntries) {
+	  fprintf(stderr, "%s %s \"%s\"table entries %u doesn't match count %u\n", 
+		  GetParentAtom() != NULL ? GetParentAtom()->GetType() : "", 
+		  GetName(), m_pProperties[0]->GetName(),
+		  m_pProperties[0]->GetCount(), numEntries);
+
+	  ASSERT(m_pProperties[0]->GetCount() == numEntries);
+	}
 
 	for (u_int32_t i = 0; i < numEntries; i++) {
 		WriteEntry(pFile, i);
@@ -858,7 +912,10 @@ void MP4DescriptorProperty::Dump(FILE* pFile, u_int8_t indent,
 
 	if (m_name) {
 		Indent(pFile, indent);
-		fprintf(pFile, "%s\n", m_name);
+		if (index != 0) 
+		  fprintf(pFile, "%s[%u]\n", m_name, index);
+		else
+		  fprintf(pFile, "%s\n", m_name);
 		indent++;
 	}
 
